#!/bin/bash

# Purpose:
#   Checks internal and external links for all markdown files (.md) of the repository.
#   For link checking, we use html-proofer-plugin, which is configured in mkdocs.yml.
#   By default the link checker only inspects internal links. It will inspect internal and external
#   links, when the env. variable ENABLED_HTMLPROOFER_EXTERNAL_URLS is set to true.

set -eo pipefail

export ENABLED_HTMLPROOFER=true

function usage(){
cat << EOF
$0 [options]
Check links in documentation.

Options:
  -a    Check internal and external links (default: internal links only)
  -h    Show help message
EOF
}

while getopts ":a" option; do
  case ${option} in
    a)
      export ENABLED_HTMLPROOFER_EXTERNAL_URLS=true
      echo "Info: Checking internal and external links. Might take some time."
      ;;
    h)
      usage
      exit;;
    \?)
      echo "Error: Invalid option."
      usage
      exit;;
  esac
done

# Switch for wikiscript
if [ -d "doc.zih.tu-dresden.de" ]; then
  cd doc.zih.tu-dresden.de
fi

mkdocs build
rm -rf public
